@ ------------------------------ ch5_dist.g ----------------------------

tests the distribution functions: 
i)  discretization of the invariant distribution    (ch5_disf.g)
ii) discretization of the invariant density function 
iii) Monte Carlo simulation
iv) functional approximation

input: asset grid a, optimal-next period a'=aopt from ch5_disf.g

please adjust the save and the load pathes!

-------------------------------------------------------------------------@


new;
clear all;
cls;
library nlsys,pgraph;
#include nlsys.ext;
nlset; graphset;
__output=0;
load path="d:\\tbk\\tb5\\g\\ch5";
save path="d:\\tbk\\tb5\\g\\ch5";


@ ----------------------------------------------------------------

Step 1: Parameterization and Initialization

----------------------------------------------------------------- @

@load a,aopt,ag=agdis;@
load a,aopt;

title("optimal savings");
xy(a,aopt-a);
wait;
na=rows(a);
nk=303;
tolg=0.0000001;
amax=a[na];
amin=a[1];
amin1=amin-1; /* lower bound for integration of exponential function */
amax1=amax+1;   
agstep=(amax-amin)/(nk-1);
ag=seqa(amin,agstep,nk);
ngk=300000;
kk1=meanc(ag);
pp=(0.9565~0.0435|0.5~0.5);

pp1=equivec1(pp);

nexp=2;         /* number of coefficients in functional approximation */
nint=20;        /* nint*nexp - number of nodes for Gauss-Chebyshev quadrature */
psi=0.5;        /* update of abare, avare, afold, .. */
tol=0.000001;

/* initialization of exponential function */
af=zeros(nexp+1,2);


@ ---------------------

Method 3: Monte Carlo Simulation 

------------------------ @


h0=hsec;
nf=1000;
f=zeros(nf,2);
x=rndu(nf,1);
f[.,1]=seqa(1,1,nf)/nf*(amax-amin)+amin;
y=rndu(nf,1);
y=(y.<=pp1[1]); /* 1 -- employed, 2 -- unemployed */
y=y+1;
f[.,2]=y;wait;
i=0;
crit=1;
kk1=meanc(f[.,1]);
kkit3=zeros(ngk,1);

j=0;
do until j==ngk;
        j=j+1; 
        kk0=kk1;
        z=rndu(nf,1);
        i=0;
        do until i==nf;
            i=i+1;
            k0=f[i,1];
            l=f[i,2];
            if k0<=amin; 
                k1=aopt[1,l]; 
            elseif k0>=amax;
                k1=aopt[na,l];
            else;    
                k1=lininter(a,aopt[.,l],k0);
            endif;
            f[i,1]=k1;  
            if z[i]<pp[l,1];
                f[i,2]=1;
            else;
                f[i,2]=2;
            endif;
        endo;


        nuempl=sumc(f[.,2])-nf;    /* number of unemployed people */
        xuempl=nuempl-round(pp1[2]*nf);  
        if xuempl<0;
            iuempl=0;
            do until iuempl==-xuempl;
                ii=round(rndu(1,1)*nf);
                if f[ii,2]==1;
                    f[ii,2]=2;
                    iuempl=iuempl+1;
                endif;
            endo;       
        elseif xuempl>0;
            iempl=0;
            do until iempl==xuempl;
                ii=round(rndu(1,1)*nf);
                if f[ii,2]==2;
                    f[ii,2]=1;
                    iempl=iempl+1;
                endif;
            endo;       
        endif;            

        kk1=meanc(f[.,1]);
        kkit3[j]=kk1;
        crit=meanc(abs(kk0-kk1))|stdc(f); 
        j~kk1~stdc(f)';
endo;
mcsec=etstr(hsec-h0);
cls;

    /* computation of the invariant distribution */
    f1=selif(f,f[.,2].==1); /* employed agents */
    f2=selif(f,f[.,2].==2); /* unemployed agents */
    "variance employed:";
    (stdc(f1[.,1]))^2;
    "variance unemployed: ";
    (stdc(f2[.,1]))^2;
    "time elapsed: " etstr(hsec-h0);
    "Press any key to continue...";

f1=sortc(f1,1); 
f1[.,2]=seqa(1,1,rows(f1))/nf;
f2=sortc(f2,1); 
f2[.,2]=seqa(1,1,rows(f2))/nf;
graphset;
title("distribution function employed agent");
xlabel("wealth");
xy(f1[.,1],f1[.,2]);
title("distribution function unemployed agent");
xy(f2[.,1],f2[.,2]);
kkk3=kk1;



@ -------------------------------------------

Method 4: Functional Approximation

--------------------------------------------- @


h0=hsec;
aopt1=(minc(aopt))'|aopt|(maxc(aopt))';
a2=amin1|a|amax1;
af=zeros(nexp+1,2);
afold=af;
afold[1,1]=pp1[1]/(amax1-amin);
afold[1,2]=pp1[2]/(amax1-amin);

/* computation of the initial mean and variances */
abareini=amean1(afold[1:3,1])/pp1[1];
abaruini=amean1(afold[1:3,2])/pp1[2];
avareini=avar1(afold[1:3,1],abareini)/pp1[1];
avaruini=avar1(afold[1:3,2],abaruini)/pp1[2];
"initial values: ";
"conditional mean, employed:" abareini;
"conditional mean, unemployed:" abaruini;
"conditional variance, employed: " avareini;
"conditional variance, unemployed: " avaruini;

/* solve for the coefficients */
abare=abareini;
abaru=abaruini;
avare=avareini;
avaru=avaruini;
{xf,fvp,jk,jcode}=nlsys(&mom2,afold[.,1]|afold[.,2]);
"xf: " xf';
"fvp: " fvp';
"jcode: " jcode;
af[.,1]=xf[1:nexp+1];
af[.,2]=xf[nexp+2:2*nexp+2];

y=seqa(amin1,(amax-amin1)/99,100);
z=af[1,1]*exp(af[2,1].*y+af[3,1].*y^2);
title("initialization of the distribution function");
xy(y,af[1,1]*exp(af[2,1].*y+af[3,1].*y^2));

am=zeros(ngk,2);
kkit4=zeros(ngk,1);
crit=tol+1;

j=0;
do until j==ngk;
    j=j+1;
    "iteration: " j;
    "time elapsed: " etstr(hsec-h0);
    "error: " crit;
    afe=afold[.,1];
    afu=afold[.,2];
    abareneu=amean(afe,afu,1)/pp1[1];
    "abar~abareneu: " abare~abareneu;
    abaruneu=amean(afe,afu,2)/pp1[2];
    "abaru~abaruneu: " abaru~abaruneu; 
    avareneu=avar(afe,afu,1,abare)/pp1[1];
    avaruneu=avar(afe,afu,2,abaru)/pp1[2];
    "avareneu~avaruneu: " avareneu~avaruneu;
    "avare~avaru: " avare~avaru;
    abare=psi*abare+(1-psi)*abareneu;
    abaru=psi*abaru+(1-psi)*abaruneu;
    avare=psi*avare+(1-psi)*avareneu;
    avaru=psi*avaru+(1-psi)*avaruneu;
     /* solve for the coefficients */
     {xf,fvp,jk,jcode}=nlsys(&mom2,afe|afu);
     "jcode: " jcode; cls;
     af[.,1]=xf[1:nexp+1];
     af[.,2]=xf[nexp+2:2*nexp+2];
     am[j,1]=amean1(af[.,1])/pp1[1]; am[j,2]=amean1(af[.,2])/pp1[2]; 
     kkit4[j]=am[j,1]*pp1[1]+am[j,2]*pp1[2];
     crit=meanc(abs(afold-af));
     afold=psi*af+(1-psi)*afold;    
endo;   /* j=1,..nit */

fasec=etstr(hsec-h0);
"iteration: " j;
"time elapsed: " etstr(hsec-h0);
"error: " crit;
"press any key to plot solution of projection method... ";
"mean capital stock employed/unemployed: ";
amean1(af[.,1])/pp1[1]~amean1(af[.,2])/pp1[2];
"mean of the economoy: ";
kkk4=amean1(af[.,1])+amean1(af[.,2]);
kkk4;
title("convergence of amean");
xy(seqa(1,1,ngk),am[1:ngk,1]*pp1[1]+am[1:ngk,2]*pp1[2]);
/* plotting the solution */
title("wealth distribution of the employed agent");
xlabel("asset level a");
xy(y,af[1,1]*exp(af[2,1].*y+af[3,1].*y^2));
title("wealth distribution of the unemployed agent");
xy(y,af[1,2]*exp(af[2,2].*y+af[3,2].*y^2));
xy(y,af[1,1]*exp(af[2,1].*y+af[3,1].*y^2)~af[1,2]*exp(af[2,2].*y+af[3,2].*y^2));



@ ---------------------

Method 1: Invariant Distribution by Discretization

------------------------- @

  h0=hsec;
  /* iteration to find invariant distribution */
  q1=0;
  kritg=1;
  kkit1=zeros(ngk,1);

  /* initialization of the distribution functions */

  gk=(ag-amin)/(amax-amin);
  gk=gk.*pp1';

    "computation of a'^-1";
    /* elimination of the rows with the same entry as the subsequent row */
    aopte=delif(a~aopt[.,1],aopt[.,1].==aopt[2:na,1]|0);
    aoptu=delif(a~aopt[.,2],aopt[.,2].==aopt[2:na,2]|0);

    a1=zeros(nk,2);
    l=0;
    do until l==2;
        l=l+1;
        i=0;
        do until i==nk;
            i=i+1;
            if l==1;
                if ag[i]<=aopte[1,2]; 
                    a1[i,l]=a[1]-0.1;
                elseif ag[i]>=maxc(aopte[.,2]); 
                    a1[i,l]=maxc(aopte[.,1]);
                else;
                    a1[i,l]=lininter(aopte[.,2],aopte[.,1],ag[i]);
                endif;
            else;
                if ag[i]<=aoptu[1,2]; 
                    a1[i,l]=a[1]-0.1;
                elseif ag[i]>=maxc(aoptu[.,2]); 
                    a1[i,l]=maxc(aoptu[.,1]);
                else;
                    a1[i,l]=lininter(aoptu[.,2],aoptu[.,1],ag[i]);
                endif;
            endif;
        endo;   
    endo;



  "computation of invariant distribution of wealth..";
 
  do until (q1>=ngk);
        q1=q1+1; q1~kk1;
        gk0=gk;
        gk=zeros(nk,2);
        l=0;
        do until l==2;
            l=l+1;
            i=0;
            do until i==nk;
                i=i+1;
                k0=a1[i,l];
                l1=0;
                do until l1==2;
                    l1=l1+1;
                    if k0<=ag[1]; 
                        gk[i,l1]=gk[i,l1]+0;
                    elseif k0>=ag[nk];
                        gk[i,l1]=gk[i,l1]+pp[l,l1]*pp1[l];
                    else;
                        gk[i,l1]=gk[i,l1]+lininter(ag,gk0[.,l],k0)*pp[l,l1];
                    endif;
                endo;
            endo;
        endo;
    
        kk1=(gk[1,1]+gk[1,2])*ag[1];
        F1x=(gk[2:nk,1]+gk[2:nk,2])-(gk[1:nk-1,1]+gk[1:nk-1,2]);
        F1e=gk[2:nk,1]-gk[1:nk-1,1];
        F1u=gk[2:nk,2]-gk[1:nk-1,2];
        ag1=(ag[2:nk]+ag[1:nk-1])/2;
        kk1=kk1+F1x'*ag1;
        kkit1[q1]=kk1;
        gk=gk/(gk[nk,1]+gk[nk,2]);
        kritg=sumc(abs(gk0-gk)); cls;
  endo;   /* q1=1,.., invariant distribution */


distsec=etstr(hsec-h0);
"time elapsed: " etstr(hsec-h0);
"Press any key to continue...";

title("invariant distribution");
xlabel("wealth a");
ylabel("");
xy(ag,gk);

"mean capital stock: ";
kk1=(gk[1,1]+gk[1,2])*ag[1];

F1x=(gk[2:nk,1]+gk[2:nk,2])-(gk[1:nk-1,1]+gk[1:nk-1,2]);
F1e=gk[2:nk,1]-gk[1:nk-1,1];
F1u=gk[2:nk,2]-gk[1:nk-1,2];


ag1=(ag[2:nk]+ag[1:nk-1])/2;

kk1e=gk[1,1]*ag[1];
kk1u=gk[1,2]*ag[1];

kk1=kk1+F1x'*ag1;
kk1e=kk1e+f1e'*ag1;
kk1u=kk1u+f1u'*ag1;

kkk1=kk1;
kk1;

"mean capital stock employed~unemployed";
kk1e~kk1u;

/* computation of the variance */
sigmae=gk[1,1]*(ag[1]-kk1e)^2;
sigmau=gk[1,2]*(ag[1]-kk1u)^2;
i=0;
do until i==nk-1;
    i=i+1;
    sigmae=sigmae+F1e[i]*(ag1[i]-kk1e)^2;
    sigmau=sigmau+F1u[i]*(ag1[i]-kk1u)^2;
endo;
sigmae=sigmae/pp1[1];
sigmau=sigmau/pp1[2];
"variance employed~unemployed: ";
sigmae~sigmau;
    
ag11=ag[1]|ag1;
fj=(gk[1,1]+gk[1,2])|F1x;
hj=zeros(nk,1);
hj[1]=fj[1]*ag11[1]/kk1;
gini=1-hj[1]*fj[1];
d=1;
do until d==nk;
    d=d+1;
    hj[d]=hj[d-1]+fj[d]*ag11[d]/kk1;
    gini=gini-(hj[d]+hj[d-1])*fj[d];
endo;
"gini= " gini;
title("invariant density function, computed with F(.)");
xy(ag1,F1e~F1u);
title("convergence of the mean wealth");
xy(seqa(1,1,ngk),kkit1);

cls;

@ ----------------

Method 2: Discretization of the invariant density 

--------------------- @

 /* iteration to find invariant distribution */
  q1=0;
  kritg=1;
  kkit2=zeros(ngk,1);
  
  /* initialization of the distribution functions */
 
      gk=ones(nk,2)/nk;
      gk=gk.*pp1';
  
  "computation of invariant distribution of wealth..";
  do until (q1>=ngk);
    q1=q1+1; q1~sumc(kk1);
    gk0=gk;
    gk=zeros(nk,2);
    l=0;
    do until l==2;
        l=l+1;
        i=0;
        do until i==nk;
            i=i+1;
            k0=ag[i];
            if k0<=amin; 
                k1=aopt[1,l]; 
            elseif k0>=amax;
                k1=aopt[na,l];
            else;    
                k1=lininter(a,aopt[.,l],k0);
            endif;
            if k1<=amin;
                gk[1,1]=gk[1,1]+gk0[i,l]*pp[l,1];
                gk[1,2]=gk[1,2]+gk0[i,l]*pp[l,2];
            elseif k1>=amax;
                gk[nk,1]=gk[nk,1]+gk0[i,l]*pp[l,1];
                gk[nk,2]=gk[nk,2]+gk0[i,l]*pp[l,2];
            elseif (k1>amin) and (k1<amax);
                j=sumc(ag.<=k1)+1;
                n0=(k1-ag[j-1])/(ag[j]-ag[j-1]);
                gk[j,1]=gk[j,1]+n0*gk0[i,l]*pp[l,1];
                gk[j,2]=gk[j,2]+n0*gk0[i,l]*pp[l,2];
                gk[j-1,1]=gk[j-1,1]+(1-n0)*gk0[i,l]*pp[l,1];
                gk[j-1,2]=gk[j-1,2]+(1-n0)*gk0[i,l]*pp[l,2];
            endif;
        endo;
    endo;
    kk1=gk'*ag;
    kkit2[q1]=sumc(kk1);
    
    gk=gk/sumc(sumc(gk));
    kritg=sumc(abs(gk0-gk)); cls;
  endo;   /* q1=1,.., invariant distribution */

    kk1=gk'*ag;
    kkk2=sumc(kk1);

densec=etstr(hsec-h0);
graphset;
title("invariant wealth distribution");
xy(ag,gk);
wait;
title("convergence of the capital stock");
xy(seqa(1,1,ngk),kkit2);
title("invariant wealth density employed agents");
_plegctl=1;
_plegstr="approximation of F(a,e)\000approximation of f(a,e)";

ag1=ag1|ag1[nk-1];
F1e=F1e|F1e[nk-1];
F1u=F1u|F1u[nk-1];
xy(ag1~ag,F1e~gk[.,1]);
wait;
title("invariant wealth density unemployed agents");
xy(ag1~ag,F1u~gk[.,2]);
wait;
cls;




"computational time: monte carlo~invdist~invdens~function approximation:";
mcsec;
distsec;
densec;
fasec;
"mean wealth: monte carlo~invdist~invdens~function approximation:";
kkk3~kkk1~kkk2~kkk4;

save kkit1,kkit2,kkit3,kkit4;

wait;

title("convergence of the mean");
_plegctl=1;
_plegctl={1 4 150000 290};
_plegstr="distribution\000 density\000 Function approximation";
xlabel("iteration");
ylabel("");
xy(seqa(1,1,ngk),kkit1~kkit2~kkit4);
wait;
nsub=round((ngk-10000)/100);
subs=seqa(1,100,100)|seqa(10000,nsub,98)|ngk;
kit1=kkit1[subs];
kit2=kkit2[subs];
kit3=kkit3[subs];
kit4=kkit4[subs];
xy(subs,kit1~kit2~kit4);
wait;

title("convergence of the mean");
_plegctl=1;
_plegctl={1 4 150000 290};
_plegstr="distribution\000 density\000 Monte Carlo\000 Function approximation";
xlabel("iteration");
ylabel("");
xy(seqa(1,1,ngk),kkit1~kkit2~kkit3~kkit4);
wait;

xy(subs,kit1~kit2~kit3~kit4);
wait;

y=ag;
z=af[1,1]*exp(af[2,1].*y+af[3,1].*y^2);
fact=rows(ag)/(amax-amin);
title("invariant wealth density of the employed worker");
xlabel("capital stock");
_plegctl={1 4 350 0.002};
_plegctl=0;
_plegstr="distribution function approximation\000 density function approximation\000 function approximation";
xy(ag1~ag~ag,F1e*fact~gk[.,1]*fact~z);
wait;

"rows(F1): " rows(f1);
nad=rows(f1)-rows(ag);
ag1=amin*ones(nad,1)|ag1;
ag=amin*ones(nad,1)|ag;
F2e=zeros(nad,1)|cumsumc(F1e);
z1=zeros(nad,1)|cumsumc(gk[.,1]);
z2=zeros(nad,1)|cumsumc(z);

title("invariant wealth distribution of the employed worker");
xlabel("capital stock");
_plegstr="distribution\000 density\000 Monte Carlo \000 function approximation";
_plegctl={1 4 300 0.1};
xy(ag1~ag~f1[.,1]~ag,F2e~z1~f1[.,2]~z2*pp1[1]/sumc(z));
wait;

@ --------------------------------------------------------------------------------------------


Procedures


-------------------------------------------------------------------------------------------- @
     
proc lininter(xd,yd,x);
  local j;
  j=sumc(xd.<=x');
  retp(yd[j]+(yd[j+1]-yd[j]).*(x-xd[j])./(xd[j+1]-xd[j]));
endp;


proc equivec1(p);
    local n,x;
    n=rows(p);
    p=diagrv(p,diag(p)-ones(n,1));
    p=p[.,1:n-1]~ones(n,1);
    x=zeros(n-1,1)|1;
    retp((x'*inv(p))');
endp;


proc amean(afe,afu,x);
    local n,m,z,y,sum1,y1e,y1u,inte,intu,i,l;
    n=rows(afe);
    m=nint*n;  /* m>n quadrature points */
    z=-cos((2.*seqa(1,1,m)-1)./(2*m).*pi);
    y=(z+1).*(amax-amin1)/2+amin1;
    sum1=0;
    y1e=lininter(a2,aopt1[.,1],y);  
    y1u=lininter(a2,aopt1[.,2],y);
    if n==2;  
        inte=afe[1]*exp(afe[2].*y);
        intu=afu[1]*exp(afu[2].*y);
    endif;
    if n==3;
        inte=afe[1]*exp(afe[2].*y+afe[3].*y^2);
        intu=afu[1]*exp(afu[2].*y+afu[3].*y^2);
    endif;
    l=0;
    do until l==m;
        l=l+1;
    sum1=sum1+pp[1,x]*maxc(amin|y1e[l])*inte[l]*sqrt(1-z[l]^2)+pp[2,x]*maxc(amin|y1u[l])*intu[l]*sqrt(1-z[l]^2);
    endo;     
    sum1=sum1*pi/2*(amax-amin1)/m;
    retp(sum1);
endp;

proc amean1(afx);
    local n,m,z,y,sum1,intx,i,l,j;
    n=rows(afx);
    m=nint*n;  /* m>n quadrature points */
    z=-cos((2.*seqa(1,1,m)-1)./(2*m).*pi);
    y=(z+1).*(amax-amin1)/2+amin1;
    sum1=0;
    if n==2;
        intx=afx[1]*exp(afx[2].*y);
    endif;
    if n==3;
        intx=afx[1]*exp(afx[2].*y+afx[3].*y^2);
    endif;
    l=0;
    do until l==m;
       l=l+1;
       sum1=sum1+maxc(amin|y[l])*intx[l]*sqrt(1-z[l]^2);
    endo;     
    sum1=sum1*pi/2*(amax-amin1)/m;
    retp(sum1);
endp;


proc avar1(afx,x);
    local n,m,z,y,sum1,intx,i,l,j;
    n=rows(afx);
    m=nint*n;  /* m>n quadrature points */
    z=-cos((2.*seqa(1,1,m)-1)./(2*m).*pi);
    y=(z+1).*(amax-amin1)/2+amin1;
    sum1=0;
    if n==2;
        intx=afx[1]*exp(afx[2].*y);
    endif;
    if n==3;
        intx=afx[1]*exp(afx[2].*y+afx[3].*y^2);
    endif;
    @
    j=0;
    do until j==m;
        j=j+1;
        "j~y[j]~afx[2]*y[j]~afx[3]*y[j]^2";
        j~y[j]~afx[2]*y[j]~afx[3]*y[j]^2;
        "intx[j]~exp(afx[2]*y)~exp(afx[3]*y^2)";
        intx[j]~exp(afx[2]*y[j])~exp(afx[3]*y[j]^2);
    endo;
    @
    l=0;
    do until l==m;
        l=l+1;
        sum1=sum1+(maxc(amin|y[l])-x)^2*intx[l]*sqrt(1-z[l]^2);
        "sum1~intx[l]~z[l]: " sum1~intx[l]~z[l];
    endo;     
    sum1=sum1*pi/2*(amax-amin1)/m;
    retp(sum1);
endp;

proc avar(afe,afu,x,abar);
    local n,m,z,y,sum1,y1e,y1u,inte,intu,i,l;
    n=rows(af);
    m=nint*n;  /* m>n quadrature points */
    z=-cos((2.*seqa(1,1,m)-1)./(2*m).*pi);
    y=(z+1).*(amax-amin1)/2+amin1;
    sum1=0;
    y1e=lininter(a2,aopt1[.,1],y);  
    y1u=lininter(a2,aopt1[.,2],y);
    if n==2;  
        inte=afe[1]*exp(afe[2].*y);
        intu=afu[1]*exp(afu[2].*y);
    endif;
    if n==3;
        inte=afe[1]*exp(afe[2].*y+afe[3].*y^2);
        intu=afu[1]*exp(afu[2].*y+afu[3].*y^2);
    endif;

    l=0;
    do until l==m;
        l=l+1;
        sum1=sum1+pp[1,x]*(maxc(amin|y1e[l])-abar)^2*inte[l]*sqrt(1-z[l]^2);
        sum1=sum1+pp[2,x]*(maxc(amin|y1u[l])-abar)^2*intu[l]*sqrt(1-z[l]^2);
    endo;     
    sum1=sum1*pi/2*(amax-amin1)/m;
    retp(sum1);
endp;


proc mom2(x);   /* moments computation for n=2 */
    local  n,m,z,y,sum1,sum2,afe,afu,inte,intu,l,res;
    n=rows(x)/2;
    m=nint*n;  /* m>n quadrature points */
    z=-cos((2.*seqa(1,1,m)-1)./(2*m).*pi);
    y=(z+1).*(amax-amin1)/2+amin1;
    sum1=zeros(n,1);
    sum2=zeros(n,1);
    afe=x[1:n];
    afu=x[n+1:2*n];
    inte=afe[1]*exp(afe[2].*y+afe[3].*y^2);
    intu=afu[1]*exp(afu[2].*y+afu[3].*y^2);
    l=0;
    do until l==m;
         l=l+1;
         sum1[1]=sum1[1]+inte[l]*sqrt(1-z[l]^2);
         sum2[1]=sum2[1]+intu[l]*sqrt(1-z[l]^2);
         sum1[2]=sum1[2]+inte[l]*sqrt(1-z[l]^2)*maxc(y[l]|amin);
         sum2[2]=sum2[2]+intu[l]*sqrt(1-z[l]^2)*maxc(y[l]|amin);
         sum1[3]=sum1[3]+inte[l]*sqrt(1-z[l]^2)*(maxc(y[l]|amin)-abare)^2;
         sum2[3]=sum2[3]+intu[l]*sqrt(1-z[l]^2)*(maxc(y[l]|amin)-abaru)^2;
     endo;     
     sum1=sum1*pi/2*(amax-amin1)/m;
     sum2=sum2*pi/2*(amax-amin1)/m;
     res=x;
     res[1]=(pp1[1]-sum1[1])/pp1[1];
     res[2]=(pp1[2]-sum2[1])/pp1[2];
     res[3]=(abare*pp1[1]-sum1[2])/abare;
     res[4]=(abaru*pp1[2]-sum2[2])/abaru;
     res[5]=(avare*pp1[1]-sum1[3])/avare;
     res[6]=(avaru*pp1[2]-sum2[3])/avaru;
     retp(res);
endp;

